#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-n',
                        '--lineno',
                        dest='lineno',
                        default=False,
                        action='store_true',
                        help='show line no.')
    parser.add_argument('-f', '--file', dest='tgtfile', help='file to check')
    parser.add_argument('-r',
                        '--rows',
                        dest='rows',
                        default=999,
                        type=int,
                        help='how many rows with nonascii char to be dumped')
    parser.add_argument('-a',
                        '--all',
                        dest='all',
                        action='store_true',
                        help='show lines without nonascii characters')
    parser.add_argument('-p',
                        '--pipe',
                        dest='pipe',
                        action='store_true',
                        default=False,
                        help='input from pipe')
    parser.add_argument(
        '-P',
        '--pipefile',
        dest='pipefile',
        action='store_true',
        default=False,
        help='treat each line from pipe as potential file to scan.')
    parser.add_argument('-L',
                        '--followlink',
                        dest='link',
                        action='store_true',
                        default=False,
                        help='follow link')
    parser.add_argument('-X',
                        '--debug',
                        dest='debug',
                        action='store_true',
                        default=False,
                        help='debug mode')
    args = parser.parse_args()

    if not (args.tgtfile or args.pipe or args.pipefile):
        parser.print_help()
        sys.exit(0)

    def scan_stream(INPUT, pipefile=False):
        if not INPUT:
            print("")
            return
        n = 0
        found = 0
        for ln in INPUT:
            if found >= args.rows:
                break
            ln = ln.rstrip()
            n += 1
            nonasciis = [(ix, c, str(hex(ord(c)))) for ix, c in enumerate(ln)
                         if ord(c) > 127]
            if nonasciis or args.all:
                found += 1
                if args.tgtfile and found == 1:
                    print(" -> contains nonascii")
                if args.lineno:
                    if nonasciis:
                        print("  {}|{} => {}".format(n, ln, nonasciis))
                    else:
                        print("  {}|{}".format(n, ln))
                else:
                    if nonasciis:
                        print("  {} => {}".format(ln, nonasciis))
                    else:
                        print("  {}".format(ln))
            if pipefile and os.path.isfile(ln):
                with open(ln, "r") as fin:
                    print("# scanning {}".format(ln))
                    scan_stream(fin.readlines(), pipefile=False)

        if found == 0:
            print("")

    filename = None
    INPUT = None
    if args.pipe or args.pipefile:
        INPUT = sys.stdin
    if args.tgtfile:
        nonasciis = [(ix, c, str(hex(ord(c))))
                     for ix, c in enumerate(args.tgtfile) if ord(c) > 127]
        if nonasciis:
            print("  {} ({})".format(args.tgtfile, nonasciis), end="")
        else:
            print("  {}".format(args.tgtfile), end="")
        if os.path.isfile(args.tgtfile) and (not os.path.islink(args.tgtfile)
                                             or args.link):
            with open(args.tgtfile, "r") as fin:
                INPUT = fin.readlines()
                filename = args.tgtfile
    scan_stream(INPUT, pipefile=args.pipefile)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import json
import yaml


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--infile', dest='infile', help='input file')
    parser.add_argument('-s',
                        '--sorted',
                        dest='sorted',
                        action='store_true',
                        default=False,
                        help='sort keys')
    parser.add_argument('-X',
                        '--debug',
                        dest='debug',
                        action='store_true',
                        default=False,
                        help='debug mode')
    args = parser.parse_args()

    if args.infile:
        if not os.path.isfile(args.infile):
            print("{} not exists.")
        with open(args.infile, "r") as f:
            INPUT = f.read()
    else:
        INPUT = sys.stdin.read()

    try:
        yl = yaml.load(INPUT)
    except:
        print("not invalid YAML")
        if args.debug:
            print(INPUT)
        sys.exit(-1)

    print(json.dumps(yl, sort_keys=args.sorted, indent=2))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import re
import json
import yaml


class JsonConverter:
    def __init__(self,
                 jsstr,
                 tblattr="border-collapse:collapse;",
                 sort=False,
                 sort_by_val=False,
                 sortkeywords=None,
                 keys_included=None,
                 keys_excluded=None,
                 colors=None) :
        if type(jsstr) is list or type(jsstr) is dict:
            self.__obj = jsstr
        else:
            try:
                self.__obj = json.loads(jsstr)
            except:
                print("# not invalid JSON.")
                sys.exit(0)
        if type(self.__obj) is dict :
            if keys_included :
                self.__obj = {k:v for k,v in self.__obj.items() if k in keys_included}
            if keys_excluded :
                for keys in keys_excluded :
                    klst = re.split(r"\.",keys)
                    pt = self.__obj
                    good = True
                    for k in klst[:-1] :
                        if k not in pt :
                            print("# warning. ignored invalid key series: {}".format(keys))
                            good = False
                            break
                        pt = pt[k]
                    if good :
                        del pt[klst[-1]]
        self.__sort = sort
        self.__sort_by_val = sort_by_val
        self.__tblattr = tblattr
        self.__sortkeywords = sortkeywords
        if colors :
            self.__colors = re.split(",",colors)
        else :
            self.__colors = None
        self.__lastcolor = 0

    def json2html(self):
        return self.json2html_helper(self.__obj)

    def __is_final_list(self, lst):
        if not type(lst) is list:
            return False
        for r in lst :
            if type(r) is dict or type(r) is list :
                return False
        return True

    def __is_final_tbl(self, lst):
        if not type(lst) is list:
            return False
        for r in lst:
            if not self.__is_final_dt(r):
                return False
        return True

    def __is_final_dt(self, dt):
        if not type(dt) is dict:
            return False
        for k, v in dt.items():
            if type(v) is dict or type(v) is list:
                return False
        return True

    def json2yaml(self):
        return yaml.safe_dump(self.__obj, default_flow_style=False)

    def json2html_helper(self, obj):
        out = ""
        inlist_border = 1
        if type(obj) is not list and type(obj) is not dict:
            out += str(obj)
            return out
        if self.__colors :
            c = self.__colors[self.__lastcolor]
            self.__lastcolor = (self.__lastcolor + 1) % len(self.__colors)
            colstr = "background-color:{}".format(c)
        else:
            colstr = ""
        if type(obj) is list:
            if self.__sort:
                obj = sorted(obj)
            style = "style=\"{};{}\"".format(self.__tblattr, colstr)
            if self.__is_final_list(obj):
                for r in obj :
                    r = str(r).replace("\r\n","<br>")
                    r = str(r).replace("\n","<br>")
                    out += "<li>" + str(r) + "</li>\n"
            elif self.__is_final_tbl(obj):
                out += "<table {} border={}>\n".format(style, inlist_border)
                need_switch = len(obj) > 1    
                colors = ["#FFFFFF","#EEFFEB"]
                ix = 0
                for r in obj:
                    c = colors[ix]
                    ix = 1-ix
                    for k, v in r.items():
                        if not v :
                            v = ""
                        else :
                            v = str(v).replace("\r\n","<br>")
                            v = str(v).replace("\n","<br>")
                        if need_switch :
                            out += "<tr style=\"background-color:{}\">\n".format(c)
                        else :
                            out += "<tr>\n"
                        out += "<td><b>" + str(k) + "</b></td>\n"
                        out += "<td>" + str(v) + "</td>\n"
                        out += "</tr>\n"
                    # + sepbar
                    #out += "<tr><td></td><td></td></tr>\n"
                out += "</table>\n"
            else:
                out += "<table {} border={}>\n".format(style, inlist_border)
                for o in obj:
                    out += "<tr>\n"
                    out += "<td>\n"
                    out += self.json2html_helper(o)
                    out += "</td>\n"
                    out += "</tr>\n"
                out += "</table>\n"
            return out
        if type(obj) is dict:
            if self.__sort:
                if self.__sort_by_val:
                    fs = lambda t: t[1]
                else:
                    fs = lambda t: t[0]
            else:
                fs = lambda t: t
            if self.__sortkeywords:

                def f(s):
                    stbl = {
                        s: ix
                        for ix, s in enumerate(
                            re.split(r",", self.__sortkeywords))
                    }
                    return (stbl.get(s, 999999), s)

                fs = lambda t: f(t[0])
            style = "style=\"{};{}\"".format(self.__tblattr, colstr)
            out += "<table {} border={}>\n".format(style, inlist_border)
            for k, v in sorted(obj.items(), key=fs):
                out += "<tr>\n"
                out += "<td><b>" + str(k) + "</b></td>\n"
                out += "<td>\n"
                out += self.json2html_helper(v)
                out += "</td>\n"
                out += "</tr>\n"
            out += "</table>\n"
            return out


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--infile', dest='infile', help='input file')
    parser.add_argument('-a',
                        '--table_attributes',
                        dest='attributes',
                        default="border-collapse:collapse;",
                        help='table attributes')
    parser.add_argument(
        '-s',
        '--sorted',
        dest='sorted',
        action='store_true',
        default=False,
        help='try best to make sure output is sorted. default by key.')
    parser.add_argument('-C',
                        '--colors',
                        dest='colors',
                        help='background colors to be used.')
    parser.add_argument('-k',
                        '--keywords_order',
                        dest='keyorder',
                        help='use this ordered keywords to help sort dict.')
    parser.add_argument('-I',
                        '--keys-included',
                        dest='keys_included',
                        action="append",
                        help='include these TOP level keys only')
    parser.add_argument('-E',
                        '--keys-excluded',
                        dest='keys_excluded',
                        action="append",
                        help='exclude these keys. eg. k1 or k2.k3')
    parser.add_argument('-v',
                        '--dictsortbyvalue',
                        dest='dtbyval',
                        action='store_true',
                        default=False,
                        help='with -s, sort dict by value.')
    parser.add_argument('-Y',
                        '--yaml',
                        dest='yaml',
                        action='store_true',
                        default=False,
                        help='dump in YAML format')
    parser.add_argument('-X',
                        '--debug',
                        dest='debug',
                        action='store_true',
                        default=False,
                        help='debug mode')
    args = parser.parse_args()

    if args.infile:
        if not os.path.isfile(args.infile):
            print("{} not exists.")
        with open(args.infile, "r") as f:
            INPUT = f.read()
    else:
        INPUT = sys.stdin.read()

    if args.yaml:
        print(
            JsonConverter(INPUT,
                          tblattr=args.attributes,
                          sort=args.sorted,
                          sort_by_val=args.dtbyval,
                          keys_included=args.keys_included,
                          keys_excluded=args.keys_excluded,
                          sortkeywords=args.keyorder).json2yaml())
        sys.exit(0)

    print(
        JsonConverter(INPUT,
                      tblattr=args.attributes,
                      sort=args.sorted,
                      sort_by_val=args.dtbyval,
                      sortkeywords=args.keyorder,
                      keys_included=args.keys_included,
                      keys_excluded=args.keys_excluded,
                      colors=args.colors).json2html())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import re
import jaydebeapi
from sretools import SimpleTable

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-J', '--jar', dest='jar', help='jdbc driver jar file')
    parser.add_argument('-D', '--driver', dest='driver', help='jdbc driver name')
    parser.add_argument('-U', '--url', dest='url', help='jdbc url or connection string')
    parser.add_argument('-Q', '--sql', dest='sql', help='sql statement to run')
    parser.add_argument('-w', '--maxwidth', dest='maxwidth', type=int, default=-1, help='maxwidth of column')
    parser.add_argument('-F', '--outformat', dest='format', default="", help='json,yaml,csv')
    parser.add_argument('-p', '--pivot', dest='pivot', action='store_true', default=False, help='pivot the view')
    parser.add_argument('-X', '--debug', dest='debug', action='store_true', default=False, help='debug mode')
    args = parser.parse_args()

    if not args.jar :
        print("must specify jdbc driver jar(-J).")
        sys.exit(-1)

    if not args.driver :
        print("must specify jdbc driver name(-D).")
        sys.exit(-1)

    if not args.url :
        print("must specify connection string(-U).")
        sys.exit(-1)

    if not args.sql :
        print("must specify SQL statement(-Q).")
        sys.exit(-1)

    conn = jaydebeapi.connect(args.driver, args.url, jars=args.jar)
    cur = conn.cursor()
    cur.execute(args.sql)
    rchg = cur.rowcount
    if rchg == -1 :
        try :
            data = cur.fetchall()
            hdr = [d[0] for d in cur.description]
            if args.debug :
                print("# header :",hdr)
                print("# data :",data)
            if args.pivot :
                print(SimpleTable(data=data,header=hdr).repr_pivot(),end="")
            else :
                if args.format == "json" :
                    print(SimpleTable(data=data,header=hdr).get_json(),end="")
                elif args.format == "yaml" :
                    print(SimpleTable(data=data,header=hdr).get_yaml(),end="")
                elif args.format == "csv" :
                    print(SimpleTable(data=data,header=hdr).get_csv(),end="")
                else :
                    print(SimpleTable(data=data,header=hdr,maxwidth=args.maxwidth),end="")
        except :
            pass
        finally :
            cur.close()
    else :
        print("# {} rows impacted.".format(rchg))
    conn.close()

# postgresql
# sretools-jdbc-call  -F "/usr/share/java/postgresql.jar" -D "org.postgresql.Driver" -U "jdbc:postgresql://localhost:5432/sample?user=postgres&password=postgres" -Q "select id, count(*)cnt from t1 group by id"
# id cnt 
# ------
# 1  1   
# 2  1 

# /jdbc_call.py -J ~/jdbc/db2jcc4.jar -D com.ibm.db2.jcc.DB2Driver  -U "jdbc:db2://localhost:50000/sample:user=db2inst1;password=db2inst1;" -Q "select viewschema,viewname,varchar(text) as text from syscat.views fetch first 1 rows only with ur"
# VIEWSCHEMA VIEWNAME          TEXT
# -------------------------------------------------------------------------------------------------------
# SYSIBM     CHECK_CONSTRAINTS CREATE OR REPLACE VIEW SYSIBM.CHECK_CONSTRAINTS
#                              (CONSTRAINT_CATALOG, CONSTRAINT_SCHEMA, CONSTRAINT_NAME, CHECK_CLAUSE)
#                              AS SELECT
#                              CAST(CURRENT SERVER AS VARCHAR(128)), TBCREATOR,
#                              CAST(NAME AS VARCHAR(128)), TEXT
#                              FROM SYSIBM.SYSCHECKS
#                              WHERE TYPE='C'
#                              UNION ALL
#                              SELECT CAST(CURRENT SERVER AS VARCHAR(128)), TBCREATOR,
#                              CAST(CONCAT(RTRIM(CONCAT(CHAR(CTIME), CHAR(FID) ) ),
#                              RTRIM(CHAR(COLNO)) ) AS VARCHAR(128) ),
#                              CAST(CONCAT(CONCAT('CHECK (', C.NAME), ' IS NOT NULL)') AS CLOB(64K) )
#                              FROM SYSIBM.SYSCOLUMNS C, SYSIBM.SYSTABLES T
#                              WHERE C.TBCREATOR = T.CREATOR AND C.TBNAME = T.NAME AND TYPE IN('U', 'T')
#                              AND NULLS ='N'
# 

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import re
import csv
import copy
from wcwidth import wcswidth
from collections import defaultdict
from itertools import zip_longest
from sretools import SimpleTable

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-n', '--lineno', dest='lineno', default=False, action='store_true', help='show line no.')
    parser.add_argument('-H', '--header', dest='header', help='header columns')
    parser.add_argument('-f', '--infile', dest='infile', help='input file')
    parser.add_argument('-C', '--dump-column', dest='dumpcols', help='only print columns indentified by index numbers.')
    parser.add_argument('-s', '--sortby', dest='sortby', help='column id starts with 0.')
    parser.add_argument('-d', '--delimiter', dest='delimiter', default=",", help='char to seperate columns')
    parser.add_argument('-q', '--quotechar', dest='quotechar', default="\"", help='char to seperate columns')
    parser.add_argument('-w', '--maxwidth', dest='maxwidth', type=int, default=-1, help='max col width when print in console, min==20')
    parser.add_argument('-p', '--pivot', dest='pivot', action='store_true', default=False, help='pivot wide tables.')
    parser.add_argument('-F', '--format', dest='format', help='other than console table, json,yaml or csv')
    parser.add_argument('-X', '--debug', dest='debug', action='store_true', default=False, help='debug mode')
    args = parser.parse_args()

    header = list()
    data = list()
    if args.lineno:
        header.append('#')
    headered = False
    if args.header :
        header += re.split(args.delimiter,args.header)
        headered = True
    if not args.infile : 
        INPUT = sys.stdin
    else :
        INPUT = open(args.infile,"r",newline='') 
    #with open(args.infile,"r",newline='') as f :
    if True :
        cr = csv.reader(INPUT, delimiter=args.delimiter, quotechar=args.quotechar)
        if not headered :
            header += next(cr)
            headered = True
        for r in cr :
            data.append(r)
    INPUT.close()

    if args.sortby:
        def fsort(x):
            v = list()
            for i in re.split(r",", args.sortby):
                v0 = x[int(i)] or ""
                if re.match(r"^\d+(\.\d*)*$",v0) :
                    v.append(float(v0))
                else :
                    v.append(0)
                v.append(v0)
            return v
        data = sorted(data, key=fsort)
    if args.pivot:
        print(
            SimpleTable(header=header, data=data,
                        cols=args.dumpcols).repr_pivot())
    else:
        t = SimpleTable(header=header, data=data, cols=args.dumpcols,maxwidth=args.maxwidth)
        if args.format == "json" :
            print(t.get_json(),end="")
        elif args.format == "yaml" :
            print(t.get_yaml(),end="")
        elif args.format == "csv" :
            print(t.get_csv(),end="")
        else :
            print(t,end="")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import re
import jaydebeapi
import socket
import psutil
import traceback
import json
import time
#from multiprocessing import Process as Task
from threading import Thread as Task
from sretools import SimpleTable

class DBX :
    def __init__(self,jar,driver,url) :
        self.__jar = jar
        self.__driver = driver
        self.__url = url
        self.__conn = None
        self.__cursor = None
    def close(self) :
        try :
            self.__cursor.close()
            self.__conn.close()
        except :
            pass
    def get_connection(self) :
        if not (self.__conn and self.__cursor) :
            self.__conn = jaydebeapi.connect(self.__driver, self.__url, jars=self.__jar)
            self.__cursor = self.__conn.cursor()
    def run_sql(self,sql) :
        self.__cursor.execute(sql)
        rchg = self.__cursor.rowcount
        result = dict()
        result["sql"] = sql
        result["rows_impacted"] = rchg
        if rchg == -1 :
                data = list()
                for row in  self.__cursor.fetchall() :
                    data.append([str(c) if c is not None else "" for c in row])
                hdr = [d[0] for d in self.__cursor.description]
                result["header"] = hdr
                result["data"] = data
        return json.dumps(result)
    def run_sp(self,sp,args=list()) :
        pass


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-J', '--jar', dest='jar', help='jdbc driver jar file')
    parser.add_argument('-D', '--driver', dest='driver', help='jdbc driver name')
    parser.add_argument('-U', '--url', dest='url', help='jdbc url or connection string')
    parser.add_argument('-C', '--connect', dest='connect', action='store_true', default=False, help='force connect as server')
    parser.add_argument('-Q', '--sql', dest='sql', help='sql statement to run')
    parser.add_argument('-w', '--maxwidth', dest='maxwidth', type=int, default=-1, help='maxwidth of column')
    parser.add_argument('-F', '--outformat', dest='format', default="", help='json,yaml,csv')
    parser.add_argument('-p', '--pivot', dest='pivot', action='store_true', default=False, help='pivot the view')
    parser.add_argument('-X', '--debug', dest='debug', action='store_true', default=False, help='debug mode')
    args = parser.parse_args()

    ppid = os.getppid()
    found = False
    p = ppid
    for d in ["~/.cache","~/.cache/sretools"] :
        fulld = os.path.expanduser(d)
        if not os.path.exists(fulld) :
            os.mkdir(fulld)
    while p not in [0,1] :
        if os.path.exists(os.path.expanduser("~/.cache/sretools/.dbx.{}".format(p))) :
            found = True
            break
        else :
           np = psutil.Process(ppid).ppid()
           if p == np :
               break
           else :
               p = np
    if not found or args.connect :
        addr = os.path.expanduser("~/.cache/sretools/.dbx.{}".format(ppid))
        if os.path.exists(addr) :
            os.unlink(addr)
    else :
        # client only
        addr = os.path.expanduser("~/.cache/sretools/.dbx.{}".format(p))

    def clientrun() :
        try :
            if args.debug :
                print("# DBXClient {}".format(addr))
            if args.connect and args.sql :
                time.sleep(1)
            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            s.connect(addr)
            s.send(args.sql.encode())
            if args.debug :
                print("# Client sent :",args.sql)
            rsp = s.recv(1024*1024).decode()
            if args.debug :
                print("# Client got :",rsp)
            try :
                obj = json.loads(rsp)
            except :
                obj = dict()
            if args.sql != "terminate" and"header" in obj and "data" in obj :
                if args.format == "json" :
                    t = SimpleTable(header=obj["header"],data=obj["data"])
                    print(t.get_json(),end="")
                elif args.format == "yaml" :
                    t = SimpleTable(header=obj["header"],data=obj["data"])
                    print(t.get_yaml(),end="")
                elif args.format == "csv" :
                    t = SimpleTable(header=obj["header"],data=obj["data"])
                    print(t.get_csv(),end="")
                else :
                    t = SimpleTable(header=obj["header"],data=obj["data"],maxwidth=args.maxwidth)
                    if args.pivot :
                        print(t.repr_pivot(),end="")
                    else :
                        print(t,end="")
            rw = obj.get("rows_impacted",0) 
            if rw >= 0 and args.sql != "terminate" :
                print("# {} row(s) impacted.".format(rw))
        except :
            traceback.print_exc() 

    # server
    if args.connect :
        if not all([args.jar,args.driver,args.url]) :
            print("# Must specify jdbc driver jar(-J), driver(-D) and URL(-U).")
        pid = os.fork()
        # child
        if pid == 0 :
            print("# DBXServer {}@{}".format(os.getpid(),addr))
            dbx = DBX(driver=args.driver,url=args.url,jar=args.jar)
            dbx.get_connection()
            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            s.bind(addr)
            s.listen(1)
            def sqlworker(netconn,sql) :
                try : 
                    result = dbx.run_sql(sql)
                    if args.debug :
                        print("# Sqlworker: result=",result)
                    netconn.send(result.encode())
                except :
                    traceback.print_exc() 
            while True :
                try : 
                    netconn, cltaddr = s.accept()
                    sql = netconn.recv(32768).decode()
                    if args.debug :
                        print("# Eventloop: sql={}".format(sql))
                    if sql == "terminate" :
                        break
                    wk = Task(target=sqlworker,args=(netconn,sql))
                    wk.start()
                except :
                    traceback.print_exc() 
            if os.path.exists(addr) :
                os.unlink(addr)
        # parent
        elif pid > 0 :
            if args.sql :
                clientrun()
            sys.exit(0)
        else :
            print("# Error forking new process ...")
            sys.exit(-1) 
    # client
    else :
        clientrun()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import json
import yaml
from sretools import JsonConverter


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--infile', dest='infile', help='input file')
    parser.add_argument('-s',
                        '--sorted',
                        dest='sorted',
                        action='store_true',
                        default=False,
                        help='sort keys')
    parser.add_argument('-a',
                        '--table_attributes',
                        dest='attributes',
                        default="border-collapse:collapse;",
                        help='table attributes')
    parser.add_argument('-C',
                        '--colors',
                        dest='colors',
                        help='colors to be used for background')
    parser.add_argument('-I',
                        '--keys-included',
                        dest='keys_included',
                        action="append",
                        help='include these keys only')
    parser.add_argument('-E',
                        '--keys-excluded',
                        dest='keys_excluded',
                        action="append",
                        help='exclude these keys')
    parser.add_argument('-k',
                        '--keywords_order',
                        dest='keyorder',
                        help='use this ordered keywords to help sort dict.')
    parser.add_argument('-v',
                        '--dictsortbyvalue',
                        dest='dtbyval',
                        action='store_true',
                        default=False,
                        help='with -s, sort dict by value.')
    parser.add_argument('-X',
                        '--debug',
                        dest='debug',
                        action='store_true',
                        default=False,
                        help='debug mode')
    args = parser.parse_args()

    if args.infile:
        if not os.path.isfile(args.infile):
            print("{} not exists.")
        with open(args.infile, "r") as f:
            INPUT = f.read()
    else:
        INPUT = sys.stdin.read()

    try:
        yl = yaml.safe_load(INPUT)
    except:
        print("not invalid YAML")
        if args.debug:
            print(INPUT)
        sys.exit(-1)

    print(
        JsonConverter(jsstr=yl,
                      tblattr=args.attributes,
                      sort=args.sorted,
                      sort_by_val=args.dtbyval,
                      sortkeywords=args.keyorder,
                      keys_included=args.keys_included,
                      keys_excluded=args.keys_excluded,
                      colors=args.colors).json2html())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Yonghang Wang

import sys
import argparse
import os
import re
import csv
import copy
from wcwidth import wcswidth
from collections import defaultdict
from itertools import zip_longest

class SimpleTable:
    def __init__(self, data=None, header=None, cols=None, maxwidth=-1):
        self.__maxwidth = int(maxwidth)
        if cols and re.search(",", cols):
            ncols = len(header)
            xmap = [int(i) for i in re.split(r",", cols) if int(i) < ncols]
            self.__header = [header[i] for i in xmap]
            self.__data = list()
            for d in data:
                self.__data.append([d[i] for i in xmap])
        else:
            self.__header = header or list()
            self.__data = data or list()
        self.__num_of_cols = len(self.__header)

    def import_csv(self, csvfile, header=True):
        pass

    def __len__(self):
        return len(self.__data)

    def __getitem__(self, pos):
        return self.__data[pos]

    def __add__(self, other):
        if self.__num_of_cols != other.__num_of_cols:
            return None
        return SimpleTable(self.__data + other.__data, self.__header)

    def __iadd__(self, other):
        if self.__num_of_cols != other.__num_of_cols:
            return None
        self.__data += other.__data
 
    def get_json(self) :
        import json
        tbl = list()
        for r in self.__data :
            row = dict()
            for ix, val in enumerate(r) :
                row[self.__header[ix]] = val
            tbl.append(row)
        return json.dumps(tbl,indent=2)

    def get_yaml(self) :
        import yaml
        tbl = list()
        for r in self.__data :
            row = dict()
            for ix, val in enumerate(r) :
                row[self.__header[ix]] = val
            tbl.append(row)
        return yaml.safe_dump(tbl,default_flow_style=False)

    def get_csv(self) :
        import io
        si = io.StringIO()
        w = csv.writer(si)
        if self.__header :
            w.writerow(self.__header)
        for r in self.__data :
            w.writerow(r)
        return si.getvalue()

    def repr_pivot(self):
        maxwidth = max([wcswidth(h) for h in self.__header])
        fmtstr = "{:" + str(maxwidth) + "} : {}"
        res = ""
        for r, row in enumerate(self.__data):
            if r > 0:
                res += "--\n"
            for i, c in enumerate(row):
                res += fmtstr.format(self.__header[i], c) + "\n"
        return res

    def __wcswidth_x(self,s):
        res = 0
        for ln in s.splitlines() :
            wclen = wcswidth(ln)
            if wclen > res :
                res = wclen
        return res

    def __splitstring(self,s,maxwidth) :
        result = list()
        for ln in s.splitlines() :
            if wcswidth(ln) <= maxwidth :
                result.append(ln)
            else :
                left = ln
                while left :
                    start = 0
                    end = min(maxwidth,len(left))
                    while wcswidth(left[start:end]) > maxwidth :
                        end -= 1
                    result.append(left[start:end])
                    left = left[end:]
        return result

    def __splitrow(self,row) :
        if self.__maxwidth < 20 :
            return zip_longest(*[c.splitlines() for c in row],fillvalue="")
        else :
            return zip_longest(*[self.__splitstring(str(c),self.__maxwidth) for c in row],fillvalue="")

    def __repr__(self):
        width = [0 for _ in range(len(self.__header))]
        for row in self.__data + [self.__header]:
            for ix, col in enumerate(row):
                if not col :
                    col = ""
                if ix > len(width) - 1:
                    break
                wclen = self.__wcswidth_x(str(col)) 
                if wclen > width[ix]:
                    width[ix] = wclen
        if self.__maxwidth >= 20 :     
            width = [min(w,self.__maxwidth) for w in width]
        twidth = copy.copy(width)
        for ix, w in enumerate(twidth):
            twidth[ix] = w - wcswidth(str(self.__header[ix])) + len(str(self.__header[ix]))
        fmtstr = "".join(["{:" + str(l + 1) + "}" for l in twidth])
        res = fmtstr.format(*self.__header).rstrip() + "\n"
        res += ('-' * (sum(width) + len(width) - 1)) + "\n"
        for r in self.__data:
            row = [str(c) if c else "" for c in r]
            if len(row) < len(width):
                row.extend([""] * (len(width) - len(row)))
            for t in self.__splitrow(row) :
                twidth = copy.copy(width)
                for ix, w in enumerate(twidth):
                    twidth[ix] = w - wcswidth(str(t[ix])) + len(str(t[ix]))
                fmtstr = "".join(["{:" + str(l + 1) + "}" for l in twidth])
                res += fmtstr.format(*t).rstrip() + "\n"
        return res

def tokenize(s):
    offset = 0
    tokens = list()
    while True:
        m = re.search(r"\S+\s*", s[offset:])
        if m:
            tokens.append(
                [offset + m.start(), offset + m.end(),
                 m.group().rstrip()])
            offset += m.end()
        else:
            break
    tokens[-1][1] = -1
    return tokens


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-n',
                        '--lineno',
                        dest='lineno',
                        default=False,
                        action='store_true',
                        help='show line no.')
    parser.add_argument('-H', '--header', dest='header', help='header columns')
    parser.add_argument('-f', '--infile', dest='infile', help='input file')
    parser.add_argument('-c',
                        '--column',
                        action='append',
                        dest='column',
                        help='column names. used when there'
                        're spaces within.')
    parser.add_argument(
        '-C',
        '--dump-column',
        dest='dumpcols',
        help='only print columns indentified by index numbers.')
    parser.add_argument('-s',
                        '--sortby',
                        dest='sortby',
                        help='column id starts with 0.')
    parser.add_argument('-b',
                        '--sep-char',
                        dest='sepchar',
                        default="\s+",
                        help='char to seperate columns')
    parser.add_argument('-w',
                        '--maxwidth',
                        dest='maxwidth',
                        type=int,
                        default=-1,
                        help='max col width when print in console, min==20')
    parser.add_argument(
        '-t',
        '--table',
        dest='table',
        action='store_true',
        default=False,
        help='input preformatted by spaces. header should not include spaces.')
    parser.add_argument('-p',
                        '--pivot',
                        dest='pivot',
                        action='store_true',
                        default=False,
                        help='pivot wide tables.')
    parser.add_argument('-F', '--format', dest='format', help='other than console table, json,yaml or csv')
    parser.add_argument('-X',
                        '--debug',
                        dest='debug',
                        action='store_true',
                        default=False,
                        help='debug mode')
    args = parser.parse_args()

    colsdict = defaultdict()
    colsdict_revert = defaultdict()
    if args.column:
        for c in args.column:
            oc = c
            mc = re.sub("\s", "_", oc)
            colsdict[oc] = mc
            colsdict_revert[mc] = oc
    if args.header and colsdict:
        for c, nc in colsdict.items():
            args.header = re.sub(c, nc, args.header)
    xheader = None
    header = list()
    if args.lineno:
        header.append('#')
    if args.header:
        mhdr = args.header
        if not args.table:
            header.extend(re.split(r"{}".format(args.sepchar), mhdr))
        else:
            xheader = tokenize(mhdr)
            header = [v.rstrip() for (s, t, v) in xheader]
    data = list()
    lno = 0
    INPUT = sys.stdin
    if args.infile:
        with open(args.infile, "r") as f:
            INPUT = f.readlines()
    for ln in INPUT:
        if not ln or not re.search(r"\S+",ln):
            continue
        if not header:
            for c, nc in colsdict.items():
                ln = re.sub(c, nc, ln)
        if not args.table:
            arr = re.split(r"{}".format(args.sepchar), ln.rstrip())
        else:
            if xheader:
                arr = [ln[s:t].rstrip() for (s, t, v) in xheader]
            else:
                for c, nc in colsdict.items():
                    ln = re.sub(c, nc, ln)
                xheader = tokenize(ln)
                if args.lineno:
                    header = ["#"] + [v.rstrip() for (s, t, v) in xheader]
                else:
                    header = [v.rstrip() for (s, t, v) in xheader]
                continue
        if len(header) == 0 or (args.lineno and len(header) == 1):
            if not args.table:
                header.extend(arr)
        else:
            lno += 1
            if args.lineno:
                data.append([str(lno)] + arr)
            else:
                data.append(arr)
    oheader = list()
    for h in header:
        nh = colsdict_revert.get(h, h)
        oheader.append(nh)
    if args.sortby:
        def fsort(x):
            v = list()
            for i in re.split(r",", args.sortby):
                v0 = x[int(i)] or ""
                if re.match(r"^\d+(\.\d*)*$",v0) :
                    v.append(float(v0))
                else :
                    v.append(0)
                v.append(v0)
            return v
        data = sorted(data, key=fsort)
    if args.pivot:
        print(
            SimpleTable(header=oheader, data=data,
                        cols=args.dumpcols).repr_pivot())
    else:
        t = SimpleTable(header=oheader, data=data, cols=args.dumpcols,maxwidth=args.maxwidth)
        if args.format == "json" :
            print(t.get_json())
        elif args.format == "yaml" :
            print(t.get_yaml())
        elif args.format == "csv" :
            print(t.get_csv())
        else :
            print(t,end="")


if __name__ == "__main__":
    main()
